@@ -51,7 +51,7 @@ public class OneLineFormatter extends Formatter {
     };
 
     /* Timestamp format */
-    private static final String DEFAULT_TIME_FORMAT = "dd-MMM-yyyy HH:mm:ss";
+    private static final String DEFAULT_TIME_FORMAT = "dd-MMM-yyyy HH:mm:ss.SSS";
 
     /**
      * The size of our global date format cache
@@ -68,6 +68,8 @@ public class OneLineFormatter extends Formatter {
      */
     private ThreadLocal<DateFormatCache> localDateCache;
 
+    private volatile MillisHandling millisHandling = MillisHandling.APPEND;
+
 
     public OneLineFormatter() {
         String timeFormat = LogManager.getLogManager().getProperty(
@@ -86,12 +88,31 @@ public class OneLineFormatter extends Formatter {
      *                   {@link java.text.SimpleDateFormat} syntax
      */
     public void setTimeFormat(final String timeFormat) {
+        final String cachedTimeFormat;
+
+        if (timeFormat.endsWith(".SSS")) {
+            cachedTimeFormat = timeFormat.substring(0,  timeFormat.length() - 4);
+            millisHandling = MillisHandling.APPEND;
+        } else if (timeFormat.contains("SSS")) {
+            millisHandling = MillisHandling.REPLACE_SSS;
+            cachedTimeFormat = timeFormat;
+        } else if (timeFormat.contains("SS")) {
+            millisHandling = MillisHandling.REPLACE_SS;
+            cachedTimeFormat = timeFormat;
+        } else if (timeFormat.contains("S")) {
+            millisHandling = MillisHandling.REPLACE_S;
+            cachedTimeFormat = timeFormat;
+        } else {
+            millisHandling = MillisHandling.NONE;
+            cachedTimeFormat = timeFormat;
+        }
+
         final DateFormatCache globalDateCache =
-                new DateFormatCache(globalCacheSize, timeFormat, null);
+                new DateFormatCache(globalCacheSize, cachedTimeFormat, null);
         localDateCache = new ThreadLocal<DateFormatCache>() {
             @Override
             protected DateFormatCache initialValue() {
-                return new DateFormatCache(localCacheSize, timeFormat, globalDateCache);
+                return new DateFormatCache(localCacheSize, cachedTimeFormat, globalDateCache);
             }
         };
     }
@@ -156,18 +177,45 @@ public class OneLineFormatter extends Formatter {
     }
 
     protected void addTimestamp(StringBuilder buf, long timestamp) {
-        buf.append(localDateCache.get().getFormat(timestamp));
-        long frac = timestamp % 1000;
-        buf.append('.');
-        if (frac < 100) {
-            if (frac < 10) {
+        String cachedTimeStamp = localDateCache.get().getFormat(timestamp);
+        if (millisHandling == MillisHandling.NONE) {
+            buf.append(cachedTimeStamp);
+        } else if (millisHandling == MillisHandling.APPEND) {
+            buf.append(cachedTimeStamp);
+            long frac = timestamp % 1000;
+            buf.append('.');
+            if (frac < 100) {
+                if (frac < 10) {
+                    buf.append('0');
+                    buf.append('0');
+                } else {
+                    buf.append('0');
+                }
+            }
+            buf.append(frac);
+        } else {
+            // Some version of replace
+            long frac = timestamp % 1000;
+            // Formatted string may vary in length so the insert point may vary
+            int insertStart = cachedTimeStamp.indexOf(DateFormatCache.MSEC_PATTERN);
+            buf.append(cachedTimeStamp.subSequence(0, insertStart));
+            if (frac < 100 && millisHandling == MillisHandling.REPLACE_SSS) {
                 buf.append('0');
-                buf.append('0');
-            } else {
+                if (frac < 10) {
+                    buf.append('0');
+                }
+            } else if (frac < 10 && millisHandling == MillisHandling.REPLACE_SS) {
                 buf.append('0');
             }
+            buf.append(frac);
+            if (millisHandling == MillisHandling.REPLACE_SSS) {
+                buf.append(cachedTimeStamp.substring(insertStart + 3));
+            } else if (millisHandling == MillisHandling.REPLACE_SS) {
+                buf.append(cachedTimeStamp.substring(insertStart + 2));
+            } else {
+                buf.append(cachedTimeStamp.substring(insertStart + 1));
+            }
         }
-        buf.append(frac);
     }
 
 
@@ -250,4 +298,13 @@ public class OneLineFormatter extends Formatter {
             super.println(x);
         }
     }
+
+
+    private static enum MillisHandling {
+        NONE,
+        APPEND,
+        REPLACE_S,
+        REPLACE_SS,
+        REPLACE_SSS,
+    }
 }
